package com.huanshao.bmap_flutter.method_channel_handlers

import com.baidu.mapapi.map.BaiduMap
import com.baidu.mapapi.map.MapStatus
import com.baidu.mapapi.search.geocode.*
import com.huanshao.bmap_flutter.Consts
import io.flutter.plugin.common.EventChannel

object MapStatusChangeListener: EventChannel.StreamHandler, BaiduMap.OnMapStatusChangeListener, OnGetGeoCoderResultListener {
    var sink: EventChannel.EventSink? = null
    private lateinit var mCoder: GeoCoder

    override fun onListen(arguments: Any?, events: EventChannel.EventSink) {
        sink = events
        mCoder = GeoCoder.newInstance()
        Consts.mapView.setOnMapStatusChangeListener(this)
        mCoder.setOnGetGeoCodeResultListener(this)
    }

    override fun onCancel(arguments: Any?) {
        Consts.mapView.setOnMapStatusChangeListener(null)
        mCoder.destroy()
    }

    override fun onMapStatusChangeStart(p0: MapStatus?) {
    }

    override fun onMapStatusChangeStart(p0: MapStatus?, p1: Int) {
        if(p1 == 1 && Consts.getPoiWhenMapStatusChange){
            Consts.currentKeyword = ""
        }
    }

    override fun onMapStatusChange(p0: MapStatus?) {
    }

    override fun onMapStatusChangeFinish(p0: MapStatus) {
        if(Consts.getPoiWhenMapStatusChange){
            sink?.success(true)
            mCoder.reverseGeoCode(ReverseGeoCodeOption().location(p0.target).newVersion(1))
        }

        Consts.getPoiWhenMapStatusChange = true

        if (Consts.currentKeyword.isEmpty()){
            sink?.success("clear-keyword")
        }
    }

    override fun onGetGeoCodeResult(p0: GeoCodeResult) {

    }

    override fun onGetReverseGeoCodeResult(p0: ReverseGeoCodeResult?) {
        sink?.success(false)
        if(p0 == null || p0.poiList == null) {
            sink?.success("[]")
            return
        }
        var resStr = "["
        for (item in p0.poiList){
            resStr += "{\"name\": \"${item.name}\", \"address\": \"${item.address}\", \"latitude\": ${item.location.latitude}, \"longitude\": ${item.location.longitude}},"
        }

        resStr = resStr.removeRange(resStr.length - 1, resStr.length)

        resStr += "]"

        sink?.success(resStr)
    }
}