package com.huanshao.bmap_flutter.method_channel_handlers

import android.content.Context
import com.baidu.location.BDAbstractLocationListener
import com.baidu.location.BDLocation
import com.baidu.location.LocationClient
import com.baidu.location.LocationClientOption
import com.baidu.mapapi.map.MapStatusUpdateFactory
import com.baidu.mapapi.map.MyLocationData
import com.baidu.mapapi.model.LatLng
import com.huanshao.bmap_flutter.Consts
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

class PickLocationGetLocationHandler(val context: Context) : MethodChannel.MethodCallHandler,
    BDAbstractLocationListener() {
    private var mLocationClient: LocationClient = LocationClient(context)
    var start = false;

    init {
        val options = LocationClientOption()
        options.setCoorType("GCJ02")
        options.isOpenGps = true
        options.setIsNeedAddress(true)
        options.setIsNeedLocationDescribe(true)
        options.setIsNeedLocationPoiList(true)
        mLocationClient.locOption = options
        options.scanSpan = 1000
        mLocationClient.registerLocationListener(this)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "get-location" -> getLocation()
            "set-center" -> setCenter(call, result)
            "stop" -> {
                mLocationClient.stop()
                start = false
            }
            else -> result.notImplemented()
        }
    }

    override fun onReceiveLocation(p0: BDLocation) {
        Consts.currentCity = p0.city
        val myLocationData = MyLocationData.Builder().direction(p0.direction).latitude(p0.latitude)
            .longitude(p0.longitude).accuracy(p0.radius).build()
        Consts.mapView.setMyLocationData(myLocationData)

        if (Consts.needSetCurrentLocation) {
            Consts.mapView.animateMapStatus(
                MapStatusUpdateFactory.newLatLng(
                    LatLng(
                        p0.latitude,
                        p0.longitude
                    )
                )
            )
            Consts.mapView.animateMapStatus(MapStatusUpdateFactory.zoomTo(17f))
            Consts.pickLocationGetLocation.invokeMethod(
                "update-current-location",
                "{\"name\": \"${p0.locationDescribe}\", \"address\": \"${p0.addrStr}\", \"latitude\": ${p0.latitude}, \"longitude\": ${p0.longitude}}"
            )
        }

        Consts.needSetCurrentLocation = false
    }

    private fun getLocation() {
        if (!start) {
            Consts.mapView.isMyLocationEnabled = true
            mLocationClient.start()
            start = true;
        }
        Consts.currentKeyword = ""
        Consts.needSetCurrentLocation = true
    }

    private fun setCenter(call: MethodCall, result: MethodChannel.Result) {
        val latitude = call.argument<Double>("latitude") as Double
        val longitude = call.argument<Double>("longitude") as Double

        Consts.getPoiWhenMapStatusChange = false
        Consts.mapView.animateMapStatus(
            MapStatusUpdateFactory.newLatLng(
                LatLng(
                    latitude,
                    longitude
                )
            )
        )
    }
}