package com.huanshao.bmap_flutter.method_channel_handlers

import com.baidu.mapapi.search.sug.OnGetSuggestionResultListener
import com.baidu.mapapi.search.sug.SuggestionResult
import com.baidu.mapapi.search.sug.SuggestionSearch
import com.baidu.mapapi.search.sug.SuggestionSearchOption
import com.huanshao.bmap_flutter.Consts
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

object SearchLocationHandler: MethodChannel.MethodCallHandler, OnGetSuggestionResultListener {
    private val mSuggestionSearch = SuggestionSearch.newInstance()

    init {
        mSuggestionSearch.setOnGetSuggestionResultListener(this)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when(call.method){
            "search" -> search(call)
            "dispose" -> mSuggestionSearch.destroy()
            else -> result.notImplemented()
        }
    }

    override fun onGetSuggestionResult(p0: SuggestionResult?) {
        MapStatusChangeListener.sink?.success(false)
        if(p0 == null || p0.allSuggestions == null) {
            MapStatusChangeListener.sink?.success("[]")
            return
        }

        var resStr = "["
        for (item in p0.allSuggestions){
            if(item.pt != null && item.key.isNotEmpty()){
                resStr += "{\"name\": \"${item.key}\", \"address\": \"${item.address}\", \"latitude\": ${item.pt.latitude}, \"longitude\": ${item.pt.longitude}},"
            }
        }

        resStr = resStr.removeRange(resStr.length - 1, resStr.length)

        resStr += "]"

        MapStatusChangeListener.sink?.success(resStr)
    }

    private fun search(call: MethodCall) {
        if(MapStatusChangeListener.sink == null) return

        mSuggestionSearch.setOnGetSuggestionResultListener(this)

        val keyword = call.argument<String>("keyword") as String
        Consts.currentKeyword = keyword
        MapStatusChangeListener.sink?.success(true)
        val options = SuggestionSearchOption()
        options.city(getCurrentCity()).keyword(keyword)
        mSuggestionSearch.requestSuggestion(options)
    }

    private fun getCurrentCity(): String {
        if(Consts.currentCity != null) return Consts.currentCity!!

        return "昆明市"
    }
}