//
//  NativeView.swift
//  bmap_flutter
//
//  Created by 刘建林 on 2021/7/12.
//

import Flutter

class NativeView: NSObject, FlutterPlatformView, BMKGeneralDelegate, BMKMapViewDelegate, BMKLocationManagerDelegate, FlutterStreamHandler, BMKLocationAuthDelegate, BMKGeoCodeSearchDelegate, BMKSuggestionSearchDelegate {
    private let _frame: CGRect
    private var mapView: BMKMapView?
    private var locationManager: BMKLocationManager?
    private var userLocation: BMKUserLocation?
    private let kBinaryMessenger: FlutterBinaryMessenger?
    private var mapManager: BMKMapManager?
    private let eventChannel: FlutterEventChannel
    private let getLocationMethodChannel: FlutterMethodChannel
    private let searchLocationMethodChannel: FlutterMethodChannel
    private var start = false
    private var currentKeyword = ""
    private var needSetCurrentLocation = true
    private var getPoiWhenMapStatusChange = true
    private var rgcData: BMKLocationReGeocode?
    private var geoCodeSearch: BMKGeoCodeSearch?
    private var sink: FlutterEventSink?
    private var suggestionSearch: BMKSuggestionSearch?
    private var currentCity = "昆明"
    private var firstLocationChange = true;
    private var ak = ""
    
    
    func view() -> UIView {
        mapManager = BMKMapManager()
        mapManager!.start(ak, generalDelegate: self)
        BMKMapManager.setCoordinateTypeUsedInBaiduMapSDK(BMK_COORD_TYPE.COORDTYPE_COMMON)
        mapView = BMKMapView(frame: _frame)
        mapView!.delegate = self
        
        mapView!.isRotateEnabled = false
        mapView!.isOverlookEnabled = false
        mapView!.isChangeCenterWithDoubleTouchPointEnabled = false
        mapView?.logoPosition = BMKLogoPositionLeftTop
        mapView!.showsUserLocation = true
        mapView!.userTrackingMode = BMKUserTrackingModeHeading
        
//        开启定位
        locationManager = BMKLocationManager.init()
        userLocation = BMKUserLocation.init()
        locationManager?.delegate = self
        locationManager?.coordinateType = BMKLocationCoordinateType.GCJ02
        locationManager?.isNeedNewVersionReGeocode = true
        
//        初始化逆解析
        geoCodeSearch = BMKGeoCodeSearch.init()
        geoCodeSearch?.delegate = self
        
//        搜索
        suggestionSearch = BMKSuggestionSearch.init()
        suggestionSearch?.delegate = self
        
        
        return mapView!
    }
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger messenger: FlutterBinaryMessenger?
    ) {
        if let kArgs = args as? Dictionary<String, Any>,
           let _ak = kArgs["iosAk"] as? String{
            ak = _ak
        }
        
        _frame = frame
        kBinaryMessenger = messenger
        eventChannel = FlutterEventChannel(name: "listen-map-status-change", binaryMessenger: messenger!)
        getLocationMethodChannel = FlutterMethodChannel(name: "pick-location-get-location", binaryMessenger: messenger!)
        searchLocationMethodChannel = FlutterMethodChannel(name: "search-location", binaryMessenger: messenger!)
        
        super.init()
        let locationAuth = BMKLocationAuth.sharedInstance()
        locationAuth?.checkPermision(withKey: ak, authDelegate: self)
        eventChannel.setStreamHandler(self)
        
        getLocationMethodChannel.setMethodCallHandler(getLocationMethodChannelHandle)
        searchLocationMethodChannel.setMethodCallHandler(searchLocationMethodChannelHandle)
    }
    
//    地图区域放生变化结束
    func mapView(_ mapView: BMKMapView!, regionDidChangeAnimated animated: Bool, reason: BMKRegionChangeReason) {
        if getPoiWhenMapStatusChange && (reason == BMKRegionChangeReasonGesture || firstLocationChange){
            firstLocationChange = false
            changeLoadingStatus(loading: true)
            let option = BMKReverseGeoCodeSearchOption.init()
            option.location = mapView.centerCoordinate
            option.isLatestAdmin = true
            
            geoCodeSearch?.reverseGeoCode(option)
        }
        
    
        if currentKeyword.isEmpty && reason == BMKRegionChangeReasonGesture && getPoiWhenMapStatusChange {
            sinkMessage(msg: "clear-keyword")
        }
        
        getPoiWhenMapStatusChange = true
    }
    
//    event channel监听
    func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        sink = events
        
        return nil
    }
    
    func onCancel(withArguments arguments: Any?) -> FlutterError? {
        return nil
    }
    
    func getLocationMethodChannelHandle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "get-location":
            if !start{
                locationManager?.startUpdatingLocation()
                start = true
            }
            
            if userLocation?.location != nil{
                mapView?.setCenter(userLocation!.location.coordinate, animated: true)
                mapView?.zoomLevel = 17
                postCurrentLocation()
                needSetCurrentLocation = false
            }else{
                needSetCurrentLocation = true
            }
            
            firstLocationChange = true
            
            currentKeyword = ""
            sinkMessage(msg: "clear-keyword")
        case "stop":
            locationManager?.stopUpdatingLocation()
            locationManager?.stopUpdatingHeading()
            start = false
        case "set-center":
            getPoiWhenMapStatusChange = false
            if let args = call.arguments as? Dictionary<String, Any>,
               let latitude = args["latitude"] as? Double,
               let longitude = args["longitude"] as? Double{
                mapView?.setCenter(CLLocationCoordinate2D.init(latitude: latitude, longitude: longitude), animated: true)
            }
        default:
            result(false)
        }
    }
    
    func searchLocationMethodChannelHandle(_ call: FlutterMethodCall, result: @escaping FlutterResult){
        switch call.method {
        case "search":
            if let args = call.arguments as? Dictionary<String, Any>,
               let keyword = args["keyword"] as? String{
                let searchOption = BMKSuggestionSearchOption.init()
                searchOption.cityname = currentCity
                searchOption.keyword = keyword
                changeLoadingStatus(loading: true)
                suggestionSearch?.suggestionSearch(searchOption)
            }
        case "dispose":
            result(false)
        default:
            result(false)
        }
    }
    
//    接收当前位置更新
    func bmkLocationManager(_ manager: BMKLocationManager, didUpdate location: BMKLocation?, orError error: Error?) {
        if error == nil{
            currentCity = location?.rgcData?.city ?? "昆明"
            rgcData = location?.rgcData
            userLocation?.location = location?.location
            mapView?.updateLocationData(userLocation)
            
            if needSetCurrentLocation{
                if let coordinate = location?.location?.coordinate{
                    mapView?.setCenter(coordinate, animated: true)
                    mapView?.zoomLevel = 17
                }
                
                postCurrentLocation()
            }
            
            needSetCurrentLocation = false
        }
    }
    
    func bmkLocationManager(_ manager: BMKLocationManager, didUpdate heading: CLHeading?) {
        userLocation?.heading = heading
        mapView?.updateLocationData(userLocation)
    }
    
//    向flutter发送当前位置信息
    func postCurrentLocation() {
        if let data = rgcData,let location = userLocation?.location{
            let address = "\(String(data.country))\(String(data.province))\(String(data.city))\(String(data.district))\(String(data.town))\(String(data.street))"
            let postData = "{\"name\": \"\(String(describing: String(data.locationDescribe)))\", \"address\": \"\(address)\", \"latitude\": \(String(describing: location.coordinate.latitude)), \"longitude\": \(String(describing: location.coordinate.longitude))}"
            getLocationMethodChannel.invokeMethod("update-current-location", arguments: postData)
        }
    }
    
//    接收到逆解析结果
    func onGetReverseGeoCodeResult(_ searcher: BMKGeoCodeSearch!, result: BMKReverseGeoCodeSearchResult!, errorCode error: BMKSearchErrorCode) {
        changeLoadingStatus(loading: false)
        if(error == BMK_SEARCH_NO_ERROR){
            let poiList = result.poiList
            var resStr = "["
            if let kPoiList = poiList{
                for poi in kPoiList{
                    let strItem = "{\"name\": \"\(String(poi.name))\", \"address\": \"\(String(poi.address))\", \"latitude\": \(String(poi.pt.latitude)), \"longitude\": \(String(poi.pt.longitude))},"
                    
                    resStr += strItem
                }
                
                resStr.removeLast()
                
                resStr += "]"
                
                sinkMessage(msg: resStr)
            }
        }else{
            print("检索失败")
            sinkMessage(msg: "[]")
        }
    }
    
    func onGetSuggestionResult(_ searcher: BMKSuggestionSearch!, result: BMKSuggestionSearchResult!, errorCode error: BMKSearchErrorCode) {
        changeLoadingStatus(loading: false)
        if(error == BMK_SEARCH_NO_ERROR) {
            let poiList = result.suggestionList
            var resStr = "["
            if let kPoiList = poiList{
                for poi in kPoiList{
                    let strItem = "{\"name\": \"\(String(poi.key))\", \"address\": \"\(String(poi.address))\", \"latitude\": \(String(poi.location.latitude)), \"longitude\": \(String(poi.location.longitude))},"
                    
                    resStr += strItem
                }
                
                resStr.removeLast()
                
                resStr += "]"
                
                sinkMessage(msg: resStr)
            }
        }else{
            print("搜索检索失败")
            sinkMessage(msg: "[]")
        }
    }
    
    func changeLoadingStatus(loading: Bool) {
        if let kSink = sink{
            kSink(loading)
        }
    }
    
    func sinkMessage(msg: Any){
        if let kSink = sink{
            kSink(msg)
        }
    }
}
