part of bmap_flutter;

class _PickLocationController {
  StreamSubscription? _subscription;

  final ValueNotifier<PoiModel?> _currentLocation = ValueNotifier(null);

  final StreamController<List<PoiModel>> _streamController = StreamController();

  final ValueNotifier<bool> _loading = ValueNotifier(true);

  Timer? _timer;

  Timer? _searchDebounceTimer;

  final TextEditingController _textEditingController = TextEditingController();
  final FocusNode _focusNode = FocusNode();

  /// 注册位置信息监听
  /// 地图状态改变，点击定位图标，搜索框输入关键词
  static const EventChannel _channel =
      const EventChannel("listen-map-status-change");

  /// 请求定位，用于位置选择
  /// 首次进入执行或点击定位图标
  static const MethodChannel _methodChannel =
      const MethodChannel("pick-location-get-location");

  /// 搜索
  static const MethodChannel _searchChannel =
      const MethodChannel("search-location");

  /// 注册监听
  registerMapStatusChangeListener({bool canGetLocation = false}) {
    final stream = _channel.receiveBroadcastStream();
    _subscription = stream.listen((val) {
      if (val is bool) {
        _loading.value = val;
      } else if (val is String && val == "clear-keyword") {
        _textEditingController.text = "";
        _focusNode.unfocus();
      } else {
        final poiList = _$PoiListFromJson(val);

        _streamController.sink.add(poiList);
      }
    });

    /// 如果有定位权限则发起定位
    if (canGetLocation) {
      _methodChannel.invokeMethod("get-location");
    }

    _methodChannel.setMethodCallHandler((call) async {
      if (call.method == "update-current-location") {
        print(call.arguments);
        _currentLocation.value = PoiModel.fromJson(json.decode(call.arguments));
      }
      return true;
    });
  }

  void onClickPoiSetCenter(PoiModel poiModel) {
    _methodChannel.invokeMethod("set-center",
        {"latitude": poiModel.latitude, "longitude": poiModel.longitude});
  }

  void myLocation() {
    _timer?.cancel();

    _timer = Timer(const Duration(milliseconds: 300), () {
      _methodChannel.invokeMethod("get-location");
    });
  }

  void search(String keyword) {
    if (keyword.isEmpty) return;
    _searchDebounceTimer?.cancel();
    _searchDebounceTimer = Timer(const Duration(milliseconds: 300), () {
      _searchChannel.invokeMethod("search", {"keyword": keyword});
    });
  }

  /// 取消监听
  dispose() {
    _timer?.cancel();
    _searchDebounceTimer?.cancel();
    _subscription?.cancel();
    _streamController.close();
    _loading.dispose();
    _searchChannel.invokeMethod("dispose");
    _methodChannel.invokeMethod("stop");
    _textEditingController.dispose();
    _focusNode.dispose();
  }
}
