part of bmap_flutter;

class _PickLocationUiFramwork extends StatefulWidget {
  final String iosAk;
  final _PickLocationController pickLocationController;
  _PickLocationUiFramwork(
      {Key? key, required this.pickLocationController, required this.iosAk})
      : super(key: key);

  @override
  __PickLocationUiFramworkState createState() =>
      __PickLocationUiFramworkState();
}

class __PickLocationUiFramworkState extends State<_PickLocationUiFramwork>
    with SingleTickerProviderStateMixin {
  late final AnimationController _animationController = AnimationController(
      vsync: this, duration: const Duration(milliseconds: 225));
  late final Animation<double> _animation = Tween<double>(begin: .7, end: .35)
      .animate(CurvedAnimation(
          parent: _animationController, curve: Curves.easeInOut));

  double _lastOffset = 0.0;

  bool _mapVisible = true;

  final ScrollController _scrollController = ScrollController();

  late final FocusNode _focusNode = widget.pickLocationController._focusNode;

  PoiModel? _selected;

  @override
  void initState() {
    super.initState();

    _focusNode.addListener(() {
      if (_focusNode.hasFocus) {
        if (_animation.value == .7) {
          _animationController.forward(from: 0);
        }
      } else {
        if (_animation.value == .35 &&
            widget.pickLocationController._textEditingController.text.isEmpty) {
          _animationController.reverse(from: 1);
          _scrollController.animateTo(0,
              duration: const Duration(milliseconds: 225),
              curve: Curves.linear);
        }
      }
    });
  }

  @override
  void dispose() {
    _animationController.dispose();
    _scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () => FocusScope.of(context).requestFocus(FocusNode()),
      child: Material(
        child: LayoutBuilder(
          builder: (BuildContext context, BoxConstraints constraints) {
            return Stack(
              children: [
                Positioned(
                  child: AnimatedBuilder(
                    animation: _animation,
                    builder: (_, child) {
                      return Container(
                        height: constraints.biggest.height * _animation.value,
                        child: child,
                      );
                    },
                    child: Visibility(
                      child: BMapView(
                        iosAk: widget.iosAk,
                      ),
                      visible: _mapVisible,
                    ),
                  ),
                  left: 0,
                  right: 0,
                  top: 0,
                ),
                Positioned(
                  child: Container(
                    padding: EdgeInsets.only(
                        top: MediaQuery.of(context).padding.top + 16,
                        bottom: 12,
                        left: 12,
                        right: 12),
                    decoration: BoxDecoration(
                        gradient: LinearGradient(
                            colors: [
                          Colors.black.withOpacity(.6),
                          Colors.black.withOpacity(0)
                        ],
                            begin: Alignment.topCenter,
                            end: Alignment.bottomCenter)),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        TextButton(
                          onPressed: () {
                            setState(() {
                              _mapVisible = false;
                            });
                            Navigator.of(context).pop();
                          },
                          child: Text(
                            "取消",
                            style: TextStyle(
                                fontWeight: FontWeight.bold, fontSize: 16),
                          ),
                          style: _CancelButtonStyle(context: context),
                        ),
                        TextButton(
                          onPressed: () {
                            setState(() {
                              _mapVisible = false;
                            });
                            Navigator.of(context).pop(_selected);
                          },
                          child: Text(
                            "确定",
                            style: TextStyle(
                                fontWeight: FontWeight.bold, fontSize: 16),
                          ),
                          style: _ConfirmButtonStyle(context: context),
                        ),
                      ],
                    ),
                  ),
                  left: 0,
                  right: 0,
                  top: 0,
                ),
                Positioned(
                  child: AnimatedBuilder(
                    animation: _animation,
                    builder: (_, child) {
                      return Container(
                        height:
                            constraints.maxHeight * (1 - _animation.value) + 20,
                        decoration: BoxDecoration(
                            color: Colors.white,
                            borderRadius: BorderRadius.only(
                                topLeft: Radius.circular(20),
                                topRight: Radius.circular(20))),
                        child: child,
                      );
                    },
                    child: Column(
                      children: [
                        Container(
                          height: 35,
                          margin: EdgeInsets.symmetric(
                              vertical: 12, horizontal: 16),
                          decoration: BoxDecoration(
                            color: Colors.black.withOpacity(.05),
                            borderRadius: BorderRadius.circular(50),
                          ),
                          child: ClipRRect(
                            borderRadius: BorderRadius.circular(50),
                            child: TextField(
                              cursorWidth: 1,
                              cursorColor: Theme.of(context).primaryColor,
                              focusNode: _focusNode,
                              onChanged: widget.pickLocationController.search,
                              controller: widget.pickLocationController
                                  ._textEditingController,
                              decoration: InputDecoration(
                                hintText: "搜索地点",
                                contentPadding: EdgeInsets.symmetric(
                                    horizontal: 20, vertical: 10),
                                border: InputBorder.none,
                                fillColor: Theme.of(context)
                                    .primaryColor
                                    .withOpacity(.05),
                                filled: true,
                              ),
                            ),
                          ),
                        ),
                        AnimatedBuilder(
                          builder: (BuildContext context, Widget? child) {
                            if (widget.pickLocationController._currentLocation
                                    .value !=
                                null) {
                              final info = widget.pickLocationController
                                  ._currentLocation.value;
                              return ListTile(
                                  title: Text("当前位置·${info!.name}"),
                                  subtitle: Text(info.address),
                                  onTap: () {
                                    widget.pickLocationController
                                        .onClickPoiSetCenter(info);
                                    widget.pickLocationController._focusNode
                                        .unfocus();

                                    setState(() {
                                      _selected = info;
                                    });
                                  },
                                  trailing: Visibility(
                                    visible: _selected != null &&
                                        _selected.toString() == info.toString(),
                                    child: Icon(
                                      Icons.check,
                                      color: Theme.of(context).primaryColor,
                                    ),
                                  ));
                            }

                            return Container();
                          },
                          animation:
                              widget.pickLocationController._currentLocation,
                        ),
                        Expanded(
                            child: NotificationListener<ScrollNotification>(
                          onNotification: (e) {
                            if (_focusNode.hasFocus) {
                              _focusNode.unfocus();
                            }
                            if (e.metrics.pixels > 0 &&
                                _lastOffset == 0 &&
                                e.metrics.maxScrollExtent > 0) {
                              if (_animation.value == .7) {
                                _animationController.forward(from: 0);
                              }
                            }

                            if (e.metrics.pixels == 0 && _lastOffset > 0) {
                              if (_animation.value == .35 &&
                                  !_focusNode.hasFocus) {
                                _animationController.reverse(from: 1);
                              }
                            }

                            _lastOffset = e.metrics.pixels;
                            return false;
                          },
                          child: StreamBuilder(
                            stream: widget.pickLocationController
                                ._streamController.stream,
                            initialData: <PoiModel>[],
                            builder: (BuildContext context,
                                AsyncSnapshot<List<PoiModel>> snapshot) {
                              return GestureDetector(
                                onTap: () => widget
                                    .pickLocationController._focusNode
                                    .unfocus(),
                                child: ListView.builder(
                                  physics: ClampingScrollPhysics(),
                                  controller: _scrollController,
                                  itemBuilder: (_, index) {
                                    final item = snapshot.data![index];

                                    return ListTile(
                                      title: Text(item.name),
                                      subtitle: Text(item.address),
                                      onTap: () {
                                        widget.pickLocationController
                                            .onClickPoiSetCenter(item);
                                        widget.pickLocationController._focusNode
                                            .unfocus();

                                        setState(() {
                                          _selected = item;
                                        });
                                      },
                                      trailing: Visibility(
                                        visible: _selected != null &&
                                            _selected.toString() ==
                                                item.toString(),
                                        child: Icon(
                                          Icons.check,
                                          color: Theme.of(context).primaryColor,
                                        ),
                                      ),
                                    );
                                  },
                                  itemCount: snapshot.data!.length,
                                ),
                              );
                            },
                          ),
                        )),
                        AnimatedBuilder(
                          animation: widget.pickLocationController._loading,
                          builder: (BuildContext context, Widget? child) {
                            return Visibility(
                              child: child!,
                              visible:
                                  widget.pickLocationController._loading.value,
                            );
                          },
                          child: LinearProgressIndicator(
                            color: Theme.of(context).primaryColor,
                          ),
                        )
                      ],
                    ),
                  ),
                  left: 0,
                  right: 0,
                  bottom: 0,
                ),
                Positioned(
                  child: AnimatedBuilder(
                    animation: _animation,
                    builder: (_, child) {
                      return Container(
                        height: constraints.biggest.height * _animation.value,
                        child: child,
                      );
                    },
                    child: Visibility(
                      child: Center(
                        child: Image(
                          image: AssetImage("assets/location.png",
                              package: "bmap_flutter"),
                          width: 40,
                          height: 40,
                        ),
                      ),
                      visible: _mapVisible,
                    ),
                  ),
                  left: 0,
                  right: 0,
                  top: -20,
                ),
                Positioned(
                  child: AnimatedBuilder(
                    animation: _animation,
                    builder: (_, child) {
                      return Container(
                        height: constraints.biggest.height * _animation.value,
                        child: child,
                      );
                    },
                    child: Align(
                      alignment: Alignment.bottomRight,
                      child: Padding(
                        padding:
                            EdgeInsets.symmetric(horizontal: 24, vertical: 32),
                        child: SizedBox(
                          width: 45,
                          height: 45,
                          child: Material(
                            color: Colors.white,
                            elevation: 4,
                            borderRadius: BorderRadius.circular(45),
                            child: InkWell(
                              onTap: widget.pickLocationController.myLocation,
                              borderRadius: BorderRadius.circular(45),
                              child: Center(
                                child: Icon(Icons.my_location),
                              ),
                            ),
                          ),
                        ),
                      ),
                    ),
                  ),
                  left: 0,
                  right: 0,
                  top: 0,
                )
              ],
            );
          },
        ),
      ),
    );
  }
}
