part of bmap_flutter;

class _PickLocation extends StatefulWidget {
  final String iosAk;
  const _PickLocation({Key? key, required this.iosAk}) : super(key: key);

  @override
  __PickLocationState createState() => __PickLocationState();
}

class __PickLocationState extends State<_PickLocation>
    with WidgetsBindingObserver {
  final _pickLocationController = _PickLocationController();

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance?.addObserver(this);

    WidgetsBinding.instance?.addPostFrameCallback((timeStamp) async {
      _pickLocationController._loading.value = true;

      final hasPermission = await _checkLocationPermission();
      await Future.delayed(const Duration(milliseconds: 500));

      _pickLocationController.registerMapStatusChangeListener(
          canGetLocation: hasPermission);

      if (!hasPermission) {
        _pickLocationController._loading.value = false;
        _noPermissionAlert();
      }
    });

    Timer(const Duration(milliseconds: 500), () {
      SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle.light
          .copyWith(statusBarColor: Colors.transparent));
    });
  }

  _noPermissionAlert() async {
    final res = await showOkCancelAlertDialog(
        context: context,
        title: "无定位权限",
        message: "您当前未授予定位权限，无法获取您当前位置，点击设置授予权限或点击取消手动搜索位置",
        okLabel: "设置",
        cancelLabel: "取消");

    if (res == OkCancelResult.ok) {
      openAppSettings();
    }
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) async {
    if (state == AppLifecycleState.resumed) {
      if (await Permission.location.isGranted) {
        _pickLocationController.myLocation();
      }
    }

    if (state == AppLifecycleState.paused) {
      _PickLocationController._methodChannel.invokeMethod("stop");
    }
    super.didChangeAppLifecycleState(state);
  }

  @override
  void dispose() {
    _pickLocationController.dispose();
    WidgetsBinding.instance?.removeObserver(this);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _PickLocationUiFramwork(
      pickLocationController: _pickLocationController,
      iosAk: widget.iosAk,
    );
  }
}
